;***************************************************************************
;  Subroutinen Display
;***************************************************************************

;***************************************************************************
;  Initialisiert das Display
;
;  Scratch-Reg: TEMP_A, 
;***************************************************************************
lcd_init:
		cbi		LCD_C1_PORT,LCD_RE		; Reset auf L
		rcall	wait_5ms
		sbi		LCD_C1_PORT,LCD_RE		; Reset auf H
		rcall	wait_5ms

		ldi		TEMP_A,low(TXT_BASE)	; Text Start Adresse
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,high(TXT_BASE)
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,TXTHOME
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,low(GFX_BASE)	; Grafik Start Adresse
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,high(GFX_BASE)
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,GFXHOME
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,SPALTEN			; Text Area
		rcall	lcd_write_data			; Daten
		clr		TEMP_A
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,TXTAREA
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,SPALTEN			; Grafik Area
		rcall	lcd_write_data			; Daten
		clr		TEMP_A
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,GFXAREA
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,MODE				; Display Mode OR, Interner CharGen
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,low(CHR_OFFSET)	; Offset-Adresse CharGen
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,high(CHR_OFFSET)
		rcall	lcd_write_data			; Daten
		ldi		TEMP_A,OFFSETA
		rcall	lcd_write_cmd			; Kommando

		ldi		TEMP_A,D_MODE+D_MODE_TXT_GFX	; Text ON, Grafik ON, Cursor OFF, Blink OFF
		rcall	lcd_write_cmd			; Kommando

		rcall	lcd_write_chargen
		rcall	lcd_clr_txt
  		rcall	lcd_clr_gfx

		ret

;***************************************************************************
;  Sendet Datenfolgen zum Display
;  Daten in TEMP_A	
;  Scratch-Reg: -
;***************************************************************************
lcd_write_auto:
		rcall	lcd_status_w_auto

#ifdef _LCD_MEM_						; Display im Memory Mapped Mode
		sts		LCD_DAT,TEMP_A
		ret
#else									; Display im I/O Mode
		rjmp	lcd_daten
#endif

;***************************************************************************
;  Sendet Kommando zum Display
;  Daten in TEMP_A	
;  Scratch-Reg: -
;***************************************************************************
lcd_write_cmd:
		rcall	lcd_status

#ifdef _LCD_MEM_						; Display im Memory Mapped Mode
		sts		LCD_CMD,TEMP_A
		ret
#else									; Display im I/O Mode
		rjmp	lcd_command
#endif

;***************************************************************************
;  Sendet Daten zum Display
;  Daten in TEMP_A	
;  Scratch-Reg: -
;***************************************************************************
lcd_write_data:

#ifdef _LCD_MEM_							; Display im Memory Mapped Mode
		rcall	lcd_status
		sts		LCD_DAT,TEMP_A
		ret
#else										; Display im I/O Mode
lcd_daten:
		cbi		LCD_C2_PORT,LCD_CD			; Daten

lcd_command:
		out		LCD_D_OUT,TEMP_A			; Daten raus

		cbi		LCD_C2_PORT,LCD_CE			; Enable L
		cbi		LCD_C1_PORT,IO_WR			; Write L

		sbi		LCD_C1_PORT,IO_WR			; Write H
		sbi		LCD_C2_PORT,LCD_CE			; Enable H

		sbi		LCD_C2_PORT,LCD_CD			; Kommando

		ret
#endif

;***************************************************************************
;  Liest Status vom Display
;  	
;  Scratch-Reg: -
;***************************************************************************
lcd_status:

#ifdef _LCD_MEM_							; Display im Memory Mapped Mode	
		push	TEMP_A

lcd_status_loop_mem:
		lds		TEMP_A,LCD_CMD
		andi	TEMP_A,(1<<STAT_CMD)+(1<<STAT_RW)	; STA0 und STA1
		cpi		TEMP_A,(1<<STAT_CMD)+(1<<STAT_RW)
		brne	lcd_status_loop_mem			; weiter warten
		
		pop		TEMP_A
		ret
#else										; Display im I/O Mode
		push	TEMP_A

		clr		TEMP_A
		out		LCD_D_DIR,TEMP_A			; auf Eingang

		cbi		LCD_C2_PORT,LCD_CE			; Enable L

lcd_status_loop:
		cbi		LCD_C1_PORT,IO_RD			; Read L
		in		TEMP_A,LCD_D_IN				; Daten lesen
		sbi		LCD_C1_PORT,IO_RD			; Read H
		
		andi	TEMP_A,(1<<STAT_CMD)+(1<<STAT_RW)	; STA0 und STA1
		cpi		TEMP_A,(1<<STAT_CMD)+(1<<STAT_RW)
		brne	lcd_status_loop				; weiter warten

		sbi		LCD_C2_PORT,LCD_CE			; Enable H

		ser		TEMP_A
		out		LCD_D_DIR,TEMP_A			; Daten auf Ausgang

		pop		TEMP_A
		ret
#endif

;***************************************************************************
;  Liest Auto-Write Status vom Display
;  	
;  Scratch-Reg: -
;***************************************************************************
lcd_status_w_auto:

#ifdef _LCD_MEM_							; Display im Memory Mapped Mode	
		push	TEMP_A

lcd_status_w_loop_mem:
		lds		TEMP_A,LCD_CMD
		andi	TEMP_A,(1<<STAT_W_AUTO)		; STA3
		cpi		TEMP_A,(1<<STAT_W_AUTO)
		brne	lcd_status_w_loop_mem		; weiter warten
		
		pop		TEMP_A
		ret
#else										; Display im I/O Mode
		push	TEMP_A

		clr		TEMP_A
		out		LCD_D_DIR,TEMP_A			; auf Eingang

		cbi		LCD_C2_PORT,LCD_CE			; Enable L

lcd_status_w_auto_loop:
		cbi		LCD_C1_PORT,IO_RD			; Read L
		in		TEMP_A,LCD_D_IN				; Daten lesen
		sbi		LCD_C1_PORT,IO_RD			; Read H

		andi	TEMP_A,(1<<STAT_W_AUTO)		; STA3
		cpi		TEMP_A,(1<<STAT_W_AUTO)
		brne	lcd_status_w_auto_loop		; weiter warten

		sbi		LCD_C2_PORT,LCD_CE			; Enable L

		ser		TEMP_A
		out		LCD_D_DIR,TEMP_A			; Daten auf Ausgang

		pop	TEMP_A
		ret
#endif

;***************************************************************************
;  Schreibt Char-Daten in Display-Char-Ram
;  	
;  Scratch-Reg: TEMP_A, TEMP_B, Z
;***************************************************************************
lcd_write_chargen:
		load_p	Z,(2*charset)
		ldi		TEMP_A,low(CHR_BASE)		; Char-Adresse
		rcall	lcd_write_data
		ldi		TEMP_A,high(CHR_BASE)
		rcall	lcd_write_data
		ldi		TEMP_A,ADPSET
		rcall	lcd_write_cmd

		ldi		TEMP_A,AWRON
		rcall	lcd_write_cmd
		ldi		TEMP_B,CHR_ANZ*8			;8Byte je Char

lcd_write_chargen_loop:
		lpm		TEMP_A,Z+
		rcall	lcd_write_auto
		dec		TEMP_B
		brne	lcd_write_chargen_loop

		ldi		TEMP_A,AWROFF
		rcall	lcd_write_cmd

		ret

;***************************************************************************
;  Lscht Textbereich vom Display
;  	
;  Scratch-Reg: -
;***************************************************************************
lcd_clr_txt:
		push	TEMP_A
		push	ZH
		push	ZL

		load_p	Z,TXT_ZEILEN*SPALTEN		; Spalten x Zeilen

		ldi		TEMP_A,low(TXT_BASE)
		rcall	lcd_write_data
		ldi		TEMP_A,high(TXT_BASE)

		rjmp	lcd_clr

;***************************************************************************
;  Lscht Grafikbereich vom Display
;  	
;  Scratch-Reg: TEMP_A, Z
;***************************************************************************
lcd_clr_gfx:
		push	TEMP_A
		push	ZH
		push	ZL

		load_p	Z,GFX_ZEILEN*SPALTEN		; Spalten x Zeilen

		ldi		TEMP_A,low(GFX_BASE)
		rcall	lcd_write_data
		ldi		TEMP_A,high(GFX_BASE)

lcd_clr:
  		rcall	lcd_write_data
  		ldi		TEMP_A,ADPSET
		rcall	lcd_write_cmd
		ldi		TEMP_A,AWRON
		rcall	lcd_write_cmd

lcd_clr_gfx_loop:
		clr		TEMP_A
		rcall	lcd_write_auto
		sbiw	Z,1
		brcc	lcd_clr_gfx_loop

		ldi		TEMP_A,AWROFF
 		rcall	lcd_write_cmd 
	
		pop		ZL
		pop		ZH
		pop		TEMP_A		
		ret

;***************************************************************************
;  Schreibt Text aus Rom ins Display bis 0-Byte
;  Textzeile in YH, Textspalte in YL, Textadresse in Z	
;  Scratch-Reg: -
;***************************************************************************
lcd_write_rom_txt:
		rcall	lcd_txt_cr2adr
		mov		TEMP_A,YL
		rcall	lcd_write_data
		mov		TEMP_A,YH
		rcall	lcd_write_data
		ldi		TEMP_A,ADPSET
		rcall	lcd_write_cmd

		ldi		TEMP_A,AWRON
		rcall	lcd_write_cmd

lcd_write_rom_txt_loop:
		lpm		TEMP_A,Z+
		tst		TEMP_A						; 0-Byte?
		breq	lcd_write_rom_txt_end

		subi	TEMP_A,$20
		rcall	lcd_write_auto
		rjmp	lcd_write_rom_txt_loop

lcd_write_rom_txt_end:
		ldi		TEMP_A,AWROFF
		rcall	lcd_write_cmd

		ret

;***************************************************************************
;  Schreibt den Inhalt der Ram-Tabelle in den Grafikbereich vom Display
;  Werte werden von 0...255 auf 0...100 Scaliert und ab OSC_ZEILE geschrieben
;	
;  Scratch-Reg: TEMP_A, YH, YL, MUL_HIGH, MUL_LOW
;***************************************************************************
lcd_write_table:	
		load_p	X,TABELLE					; Tabellenzeiger			
		ldi		YL,OSZI_SPALTE				; linke Spalte
		
lcd_write_table_loop:
		ld		TEMP_A,X+					; Wert holen (Zeilennummer)

		ldi		YH,OSZI_HOEHE
		mul		YH,TEMP_A					; TEMP_A * 100 nach r1:r0
		mov		TEMP_A,MUL_HIGH				; /256, also nur High-Byte
		sbrc	MUL_LOW,7					; grer 128?
		inc		TEMP_A						; ja, dann aufrunden

		ldi		YH,OSZI_HOEHE+OSZI_ZEILE	; unterste Zeile fr Wert 0
		sub		YH,TEMP_A					; also Wert abziehen

		push	YL
		rcall	lcd_gfx_cr2adr
		pop		YL
		rcall	lcd_set_pixel
		inc		YL
		cpi		YL,OSZI_ENDE
		brne	lcd_write_table_loop
	
		ret

;***************************************************************************
;  Zeichnet eine senkrechte Linie ab Zeile/Spalte im Grafikbereich vom Display
;  YH - Zeile, YL - Spalte, TEMP_A - Lnge	
;  Scratch-Reg: -
;***************************************************************************
lcd_vert_line:
		push	TEMP_B
		push	YH
		push	YL
		
		mov		TEMP_B,TEMP_A				; Lnge
		rcall	lcd_gfx_cr2adr				; Startadresse berechnen
			
lcd_vert_line_loop:
		push	TEMP_A
		rcall	lcd_set_pixel
		pop		TEMP_A
		adiw	Z,SPALTEN
		dec		TEMP_B
		brne	lcd_vert_line_loop

		pop		ZL
		pop		ZH
		pop		TEMP_B
		ret

;***************************************************************************
;  Zeichnet eine waagerechte Linie ab Adresse im Grafikbereich vom Display
;  YH - Zeile, YL - Spalte, TEMP_A - Lnge		
;  Scratch-Reg: -
;***************************************************************************
lcd_hor_line:
		push	TEMP_B
		push	YH
		push	YL

		mov		TEMP_B,TEMP_A				; Gfx-Lnge in TEMP_B
		rcall	lcd_gfx_cr2adr				; Startadresse holen		

		push	TEMP_A						; Pixelnummer retten
		mov		TEMP_A,ZL
		rcall	lcd_write_data
		mov		TEMP_A,ZH
		rcall	lcd_write_data
		ldi		TEMP_A,ADPSET
		rcall	lcd_write_cmd				; Zeiger auf Start setzen
		pop		TEMP_A
		
		cpi		TEMP_A,FONT_SIZE-1			; Pixel = 5 -> alle 6 Bit
		breq	lcd_hor_line_spalte

		sub		TEMP_B,TEMP_A				; Einzelpixel in Spaltenstart abziehen von Lnge
		subi	TEMP_B,1					; Korrektur fr Lnge, Pixel 0 = 1 Pixel usw.

lcd_hor_line_pixel_links:					; Restpixel in Start-Spalte setzen			
		rcall	lcd_set_pixel
		dec		TEMP_A
		cpi		TEMP_A,$FF
		brne	lcd_hor_line_pixel_links			
		
		inc		YL							; Start-Spalte +1
		rcall	lcd_gfx_cr2adr				; Startadresse neu setzen		

lcd_hor_line_spalte:			
		cpi		TEMP_B,FONT_SIZE			; Lnge kleiner Spaltenbreite?
		brlo	lcd_hor_line_loop_2			; ja, nur Einzelpixel schreiben

		clr		YL							; Abzahl Spalten berechnen

lcd_hor_line_spalten_loop:		
		inc		YL							; Spaltenzhler +1
		subi	TEMP_B,FONT_SIZE			; Anzahl Pixel pro Spalte
		cpi		TEMP_B,FONT_SIZE			; weniger als eine Spalte brig?
		brge	lcd_hor_line_spalten_loop	; in TEMP_B jetzt Pixelnummer fr rechten Rest

lcd_hor_line_spalten_end:											
		ldi		TEMP_A,AWRON
		rcall	lcd_write_cmd				; Automatik ein

lcd_hor_line_loop_1:
		ser		TEMP_A						; immer mit 8 Pixel Breite laden
		rcall	lcd_write_auto
		adiw	Z,1							; nchste Spaltenadresse
		dec		YL							; nchste Spalte
		brne	lcd_hor_line_loop_1

		ldi		TEMP_A,AWROFF
		rcall	lcd_write_cmd				; Automatik aus

lcd_hor_line_loop_2:						; Restpixel rechts zeichnen
		tst		TEMP_B						; Rest 0?
		breq	lcd_hor_line_end			; ja, nichts mehr machen
			
		ldi		TEMP_A,FONT_SIZE-1			; linken Pixel in der Spalte adressieren

lcd_hor_line_pixel_rechts:
		rcall	lcd_set_pixel
		dec		TEMP_A						; nchsten Pixel rechts davon
		dec		TEMP_B						; Anzahl Restpixel
		brne	lcd_hor_line_pixel_rechts

lcd_hor_line_end:
		pop		YL
		pop		YH
		pop		TEMP_B
		ret

;***************************************************************************
;  Setzt Pixel im Grafikbereich vom Display
;  Adresse in Z, Bitnummer in TEMP_A	
;  Scratch-Reg: -
;***************************************************************************
lcd_set_pixel:
		push	TEMP_A
		push	TEMP_B

		ldi		TEMP_B,BITSET
		or		TEMP_A,TEMP_B
		push	TEMP_A

		mov		TEMP_A,ZL
		rcall	lcd_write_data
		mov		TEMP_A,ZH
		rcall	lcd_write_data
		ldi		TEMP_A,ADPSET
		rcall	lcd_write_cmd

		pop		TEMP_A
		rcall	lcd_write_cmd

		pop		TEMP_B
		pop		TEMP_A	
		ret

;***************************************************************************
;  Berechnet Text-Adresse aus Zeile + Spalte
;  Textzeile in YH, Textspalte in YL	
;  Ergebnis in Y
;  Scratch-Reg: -
;***************************************************************************
lcd_txt_cr2adr:
		push	ZH
		push	ZL

		load_p	Z,TXT_BASE					; Korrektur fr adiw am Schleifenanfang
		tst		YH							; Zeile 0 ?
		breq	lcd_txt_cr2adr_line_0
		
lcd_txt_cr2adr_loop:
		adiw	Z,SPALTEN
		dec		YH							; Zeile				
		brne	lcd_txt_cr2adr_loop			; Start Zeile berechnen

lcd_txt_cr2adr_line_0:
		add		YL,ZL
		adc		YH,ZH						; ist 0						

		pop		ZL
		pop		ZH
		ret

;***************************************************************************
;  Berechnet Grafik-Adresse aus Zeile + Spalte
;  Grafikzeile in YH, Grafikspalte in YL	
;  Ergebnis Adresse in Z, Pixel in TEMP_A
;  Scratch-Reg: -
;***************************************************************************
lcd_gfx_cr2adr:
		load_p	Z,GFX_BASE
		clr		TEMP_A						; 0
		tst		YH							; Zeile 0?
		breq	lcd_gfx_cr2adr_line_0

lcd_gfx_cr2adr_line_loop:
		adiw	Z,SPALTEN
		dec		YH
		brne	lcd_gfx_cr2adr_line_loop 	; Zeiger Anfang nchste Zeile			

lcd_gfx_cr2adr_line_0:
		ser		YH							; wegen inc am Schleifenanfang
lcd_gfx_cr2adr_spalten_loop:		
		inc		YH							; jetzt Spaltenzhler
		subi	YL,FONT_SIZE				; Anzahl Pixel
		brcc	lcd_gfx_cr2adr_spalten_loop	; nicht unter 0
			
		add		ZL,YH						; in YH Spaltennummer, in YL -Pixelnummer
		adc		ZH,TEMP_A					; 0

lcd_gfx_cr2adr_spalte_0:
		subi	YL,-FONT_SIZE				; Pixelnummer berechnen

		ldi		TEMP_A,FONT_SIZE-1			; LSB mu nach links
		sub		TEMP_A,YL					; Pixelnummer in TEMP_A
					
		ret

;***************************************************************************
;  Schreibt den Oszi-Rahmen im Textbereich vom Display
;  Text-Adresse	in Z
;  Scratch-Reg: -
;***************************************************************************
lcd_write_rahmen:
		ldi		YH,0					; Zeile
		ldi		YL,5					; Spalte

lcd_write_rahmen_loop:
		push	YH
		push	YL
		
		rcall	lcd_write_rom_txt		; Text 1 Zeichen
		
		pop		YL
		pop		YH
		inc		YH
		cpi		YH,13					; letzte Zeile		
		brne	lcd_write_rahmen_loop

lcd_write_hor_rahmen:
		load_p	Z,(2*txt_hor_rahmen)
		ldi		YH,13					; Zeile
		ldi		YL,6					; Spalte
		
		rcall	lcd_write_rom_txt

		ret
