
.include	"definitionen.asm"

;****************************** Verktor-Liste ****************************

.CSEG
                rjmp    Reset
.org INT0addr
                reti
.org INT1addr
                reti
.org ICP1addr
                reti
.org OC1Aaddr
                reti
.org OC1Baddr
                reti
.org OVF1addr
                reti
.org OVF0addr
                reti
.org SPIaddr
                reti
.org URXCaddr
                reti
.org UDREaddr
                reti
.org UTXCaddr
                reti
.org ACIaddr
                reti
.org INT2addr
				reti
.org OC0addr
				reti
.org ERDYaddr
				reti
.org SPMRaddr
				reti

;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

; Hardware initialisieren

Reset:  ldi     TEMP_A,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_A
        ldi     TEMP_A,high(RAMEND)
        out     SPH,TEMP_A

; Externer Ram ein ***************************************************

#ifdef _LCD_MEM_							; Display im Memory Mapped Mode
		ldi		TEMP_A,(1<<SRE) //+(1<<SRW10)
		out		MCUCR,TEMP_A
;		ldi		TEMP_A,(1<<SRW11)
;		out		EMCUCR,TEMP_A
;		ldi		TEMP_A,(1<<XMBK)
;		out		SFIOR,TEMP_A
#endif

; Port A *************************************************************

        ser     TEMP_A						; Alle PullUp ein
        out     PORTA,TEMP_A
#ifdef _LCD_MEM_
		clr		TEMP_A						; Alles Eingang fr ADC
#else
		ser		TEMP_A						; Alles Ausgang (Daten)	
#endif
        out     DDRA,TEMP_A

; Port B *************************************************************

        ser     TEMP_A
        out     PORTB,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
		ser		TEMP_A	
        out     DDRB,TEMP_A

; Port C *************************************************************

        ser     TEMP_A
        out     PORTC,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High

#ifdef _LCD_MEM_							; Display im Memory Mapped Mode
		ser		TEMP_A
#else										; Display im I/O Mode
		ldi		TEMP_A,(1<<LCD_CE)+(1<<LCD_CD)	
#endif

        out     DDRC,TEMP_A

; Port D *************************************************************

		ser		TEMP_A
        out     PORTD,TEMP_A                ; Alle PullUp ein bzw. Ausgang auf High
		ldi		TEMP_A,(1<<LCD_RE)+(1<<IO_RD)+(1<<IO_WR)+(1<<ADC_CS)			
        out     DDRD,TEMP_A
  
;*********************************************************************				 

/* 	Hier wird die Messwerttabelle im Ram erstellt, gibt ja noch keine echten Werte...
	Die Rumrechnerei dabei sorgt nur fr passendere Werte,
	die Sinustabelle lag irgendwo im Netz...
	Der angedachte Wertebereich wird wohl 0...99 werden,
	dann bleibt Platz fr Beschriftungen.
	Breite wei ich auch noch nicht, evtl. 224 Werte.
	Das ist alles noch sehr unklar. ;)
	X/Y-Markierungen usw. werden als Zeichensatz angelegt, damit kann dann die Grafik
	ohne Probleme gelscht und neu gezeichnet werden.
	Hor.Line und Vert. Line sind vor allem als Cursor gedacht.                       */

		load_p	Y,TABELLE					; Sinuswerte ins Ram zum Testen
		ldi		TEMP_B,(SPALTEN*FONT_SIZE)+1
		clr		TEMP_A

init_loop_0:
		load_p	Z,(2*sinus)

init_loop_1:		
		dec		TEMP_B
		breq	init_loop_end

		adiw	Z,1							; nur jeden 2. Wert	
		push	TEMP_A
		lpm		TEMP_A,Z+
		lsr		TEMP_A						; Wert / 2	 
		subi	TEMP_A,-15					; nach oben verschieben		
		st		Y+,TEMP_A
		pop		TEMP_A
		dec		TEMP_A						; nur jeden 2. Wert
		dec		TEMP_A
		brne	init_loop_1

		rjmp	init_loop_0

init_loop_end:

		rcall	lcd_init
			 
;*********************************************************************
;  Main-Loop
;*********************************************************************
; Es werden hier zur Zeit nur Testroutinen fr die Garfiksachen aufgerufen...
main:

		load_p	Z,(2*txt_vert_rahmen_oszi)
		rcall	lcd_write_rahmen			; Zeichnet Koordinatenrahmen

		ldi		YH,12						; Zeile		(Textauflsung!)
		ldi		YL,12						; Spalte 	(Textauflsung!)
		load_p	Z,(2*text_1)
;		rcall	lcd_write_rom_txt

		ldi		YH,85						; Zeile		(Grafikauflsung!)
		ldi		YL,83	 					; Spalte	(Grafikauflsung!)
		ldi		TEMP_A,67					; Lnge		(Grafikauflsung!)
;		rcall	lcd_hor_line

		ldi		YH,7						; Zeile		(Grafikauflsung!)
		ldi		YL,17						; Spalte	(Grafikauflsung!)
		ldi		TEMP_A,80					; Lnge		(Grafikauflsung!)
;		rcall	lcd_vert_line

main_end:			
		ldi		TRIGGER_L,100
		ldi		TRIGGER_H,140				; steigendes Signal
		rcall	read_adc
		rcall	lcd_clr_gfx

		ldi		YH,6						; Zeile		(Textauflsung!)
		ldi		YL,6						; Spalte 	(Textauflsung!)
		load_p	Z,(2*txt_nullinie)
		rcall	lcd_write_rom_txt

		rcall	lcd_write_table				; Wertetabelle aus Ram - Sinus

		ldi		TEMP_A,50

main_wait:
		rcall	wait_5ms
		dec		TEMP_A
		brne	main_wait

		rjmp	main_end


.include	"ADC0820.asm"
.include	"DG24128.asm"

;*********************************************************************
;  Warte 5ms
;
;  Scratch-Reg: -
;*********************************************************************

wait_5ms:
        push    ZL
        push    ZH

        load_p  Z,5000*(CYCLES_PER_US/4)
; 5 mS warten, 4 = Anzahl Befehls-Zyclen vom sbiw + brcc

d_loop:
        sbiw    ZL,1
        brne    d_loop

        pop     ZH
        pop     ZL

        ret

;***************************************************************************
;  Programm Ende 
;***************************************************************************
								 
text_1:
		.db		"Das ist ein Testtext",0

	; Achtung! Es werden $20 abgezogen vor der Ausgabe
			
txt_hor_rahmen:
		.db		$A7,$A8,$A9,$AA,$A9,$AB,$A8,$A9,$AA,$A9
		.db		$AB,$A8,$A9,$AA,$A9,$AB,$A8,$A9,$AA,$A9
		.db		$AB,$A8,$A9,$AA,$A9,$AB,$A8,$A9,$AA,$A9
		.db		$AB,$A8,$A9,$A6,0

txt_vert_rahmen_oszi:
		.db		$A0,0,$A1,0,$A2,0,$A3,0,$A4,0,$A5,0,$A1,0,$A2,0,$A3,0
		.db		$A4,0,$A5,0,$A1,0,$A2,0

txt_vert_rahmen_logik:
		.db		$AC,0,$AD,0,$AE,0,$AF,0,$B0,0,$B1,0,$B2,0,$B3,0
		.db		$AC,0,$AD,0,$AE,0,$AF,0,$B0,0

txt_nullinie:
		.db		$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4
		.db		$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4
		.db		$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4,$B4
		.db		$B4,$B4,$B4,$B4,0

sinus:
		.db 	64,65,67,68,70,72,73,75 
		.db 	76,78,79,81,82,84,85,87 
		.db 	88,90,91,92,94,95,97,98 
		.db 	99,100,102,103,104,105,107,108 
		.db 	109,110,111,112,113,114,115,116 
		.db 	117,118,118,119,120,121,121,122 
		.db 	123,123,124,124,125,125,126,126 
		.db 	126,127,127,127,127,127,127,127 
		.db 	127,127,127,127,127,127,127,127 
		.db 	126,126,126,125,125,124,124,123 
		.db 	123,122,121,121,120,119,118,118 
		.db 	117,116,115,114,113,112,111,110 
		.db 	109,108,107,105,104,103,102,100 
		.db 	99,98,97,95,94,92,91,90 
		.db 	88,87,85,84,82,81,79,78 
		.db 	76,75,73,72,70,68,67,65 
		.db 	64,62,61,59,58,56,54,53 
		.db	 	51,50,48,47,45,44,42,41 
		.db 	39,38,36,35,34,32,31,30 
		.db 	28,27,26,25,23,22,21,20 
		.db 	19,18,17,15,14,13,13,12 
		.db 	11,10,9,8,8,7,6,5 
		.db 	5,4,4,3,3,2,2,2 
		.db 	1,1,1,0,0,0,0,0 
		.db 	0,0,0,0,0,0,1,1 
		.db 	1,2,2,2,3,3,4,4 
		.db 	5,5,6,7,8,8,9,10 
		.db 	11,12,13,13,14,15,17,18 
		.db 	19,20,21,22,23,25,26,27 
		.db 	28,30,31,32,34,35,36,38 
		.db 	39,41,42,44,45,47,48,50 
		.db 	51,53,54,56,58,59,61,62 

.include	"CharSet.asm"
